#ifndef _NVLISTEN_SOCKET_H_
#define _NVLISTEN_SOCKET_H_

#include "NvSockAddr.h"
#include "NvTcpSocket.h"

class CNvTcpSocket;

interface INvListenSocketEventSink
{
	/* This will be called when a new connection is accepted */
	virtual void OnConnected(CNvTcpSocket *pSocket) = 0;
	/* When a listen scoket is closed,this will be called */
	virtual void OnClosed() = 0;
};

class CNvListenSocketEventSink : public INvListenSocketEventSink
{
public:
	CNvListenSocketEventSink(void);
	virtual ~CNvListenSocketEventSink(void);

	virtual void OnConnected(CNvTcpSocket *pSocket);
	virtual void OnClosed();
};

interface INvListenSocket
{
	/* To get m_nSocketID */
	virtual int GetSocketID() = 0;

	/* create listen socket and bind pLocalAddr*/
	virtual bool Create(CNvSockAddr *pLocalAddr) = 0;
	
	/* Wait a new connection(Listen and Accept)
	when a new connection is established,call INvListenSocketEventSink::OnConnected(...)
	*/	
	virtual bool Listen() = 0;

	/*
	  To register an event sink 
	*/
	virtual void Advise(INvListenSocketEventSink *pEventSink) = 0;

	/*
	To close listen socket
	*/

	virtual void Close() = 0;
};

class CNvListenSocket:public INvListenSocket
{
public:
	CNvListenSocket(void);
	virtual ~CNvListenSocket(void);

	virtual int GetSocketID();
	
	virtual bool Create(CNvSockAddr *pLocalAddr);
	virtual bool Listen();

	virtual void Advise(INvListenSocketEventSink *pEventSink) { m_pEventSink = pEventSink; }

	virtual void Close();

	void OnConnected(CNvTcpSocket* clientSocket);
protected:
	int m_nSocketID;
	INvListenSocketEventSink *m_pEventSink;
};
#endif